@@ -12,7 +12,6 @@ import (
 	"github.com/docker/cli/cli/command/completion"
 	"github.com/docker/cli/opts"
 	"github.com/docker/cli/opts/swarmopts"
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
 	mounttypes "github.com/docker/docker/api/types/mount"
 	"github.com/docker/docker/api/types/network"
@@ -35,9 +34,7 @@ func newUpdateCommand(dockerCLI command.Cli) *cobra.Command {
 		RunE: func(cmd *cobra.Command, args []string) error {
 			return runUpdate(cmd.Context(), dockerCLI, cmd.Flags(), options, args[0])
 		},
-		ValidArgsFunction: func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-			return CompletionFn(dockerCLI)(cmd, args, toComplete)
-		},
+		ValidArgsFunction: completeServiceNames(dockerCLI),
 	}
 
 	flags := cmd.Flags()
@@ -115,9 +112,9 @@ func newUpdateCommand(dockerCLI command.Cli) *cobra.Command {
 
 	// Add needs parsing, Remove only needs the key
 	flags.Var(newListOptsVar(), flagGenericResourcesRemove, "Remove a Generic resource")
-	flags.SetAnnotation(flagHostAdd, "version", []string{"1.32"})
+	flags.SetAnnotation(flagGenericResourcesRemove, "version", []string{"1.32"})
 	flags.Var(newListOptsVarWithValidator(ValidateSingleGenericResource), flagGenericResourcesAdd, "Add a Generic resource")
-	flags.SetAnnotation(flagHostAdd, "version", []string{"1.32"})
+	flags.SetAnnotation(flagGenericResourcesAdd, "version", []string{"1.32"})
 
 	// TODO(thaJeztah): add completion for capabilities, stop-signal (currently non-exported in container package)
 	// _ = cmd.RegisterFlagCompletionFunc(flagCapAdd, completeLinuxCapabilityNames)
@@ -140,7 +137,7 @@ func newUpdateCommand(dockerCLI command.Cli) *cobra.Command {
 		// Set a default completion function if none was set. We don't look
 		// up if it does already have one set, because Cobra does this for
 		// us, and returns an error (which we ignore for this reason).
-		_ = cmd.RegisterFlagCompletionFunc(flag.Name, completion.NoComplete)
+		_ = cmd.RegisterFlagCompletionFunc(flag.Name, cobra.NoFileCompletions)
 	})
 
 	return cmd
@@ -158,7 +155,7 @@ func newListOptsVarWithValidator(validator opts.ValidatorFctType) *opts.ListOpts
 func runUpdate(ctx context.Context, dockerCLI command.Cli, flags *pflag.FlagSet, options *serviceOptions, serviceID string) error {
 	apiClient := dockerCLI.Client()
 
-	service, _, err := apiClient.ServiceInspectWithRaw(ctx, serviceID, types.ServiceInspectOptions{})
+	service, _, err := apiClient.ServiceInspectWithRaw(ctx, serviceID, swarm.ServiceInspectOptions{})
 	if err != nil {
 		return err
 	}
@@ -203,7 +200,7 @@ func runUpdate(ctx context.Context, dockerCLI command.Cli, flags *pflag.FlagSet,
 		}
 	}
 
-	updateOpts := types.ServiceUpdateOptions{}
+	updateOpts := swarm.ServiceUpdateOptions{}
 	if serverSideRollback {
 		updateOpts.Rollback = "previous"
 	}
@@ -257,9 +254,9 @@ func runUpdate(ctx context.Context, dockerCLI command.Cli, flags *pflag.FlagSet,
 		}
 		updateOpts.EncodedRegistryAuth = encodedAuth
 	case clientSideRollback:
-		updateOpts.RegistryAuthFrom = types.RegistryAuthFromPreviousSpec
+		updateOpts.RegistryAuthFrom = swarm.RegistryAuthFromPreviousSpec
 	default:
-		updateOpts.RegistryAuthFrom = types.RegistryAuthFromSpec
+		updateOpts.RegistryAuthFrom = swarm.RegistryAuthFromSpec
 	}
 
 	response, err := apiClient.ServiceUpdate(ctx, service.ID, service.Version, *spec, updateOpts)
@@ -584,7 +581,7 @@ func addGenericResources(flags *pflag.FlagSet, spec *swarm.TaskSpec) error {
 		spec.Resources.Reservations = &swarm.Resources{}
 	}
 
-	values := flags.Lookup(flagGenericResourcesAdd).Value.(*opts.ListOpts).GetAll()
+	values := flags.Lookup(flagGenericResourcesAdd).Value.(*opts.ListOpts).GetSlice()
 	generic, err := ParseGenericResources(values)
 	if err != nil {
 		return err
@@ -618,7 +615,7 @@ func removeGenericResources(flags *pflag.FlagSet, spec *swarm.TaskSpec) error {
 		spec.Resources.Reservations = &swarm.Resources{}
 	}
 
-	values := flags.Lookup(flagGenericResourcesRemove).Value.(*opts.ListOpts).GetAll()
+	values := flags.Lookup(flagGenericResourcesRemove).Value.(*opts.ListOpts).GetSlice()
 
 	m, err := buildGenericResourceMap(spec.Resources.Reservations.GenericResources)
 	if err != nil {
@@ -639,7 +636,7 @@ func removeGenericResources(flags *pflag.FlagSet, spec *swarm.TaskSpec) error {
 
 func updatePlacementConstraints(flags *pflag.FlagSet, placement *swarm.Placement) {
 	if flags.Changed(flagConstraintAdd) {
-		values := flags.Lookup(flagConstraintAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagConstraintAdd).Value.(*opts.ListOpts).GetSlice()
 		placement.Constraints = append(placement.Constraints, values...)
 	}
 	toRemove := buildToRemoveSet(flags, flagConstraintRemove)
@@ -686,7 +683,7 @@ func updatePlacementPreferences(flags *pflag.FlagSet, placement *swarm.Placement
 
 func updateContainerLabels(flags *pflag.FlagSet, field *map[string]string) {
 	if *field != nil && flags.Changed(flagContainerLabelRemove) {
-		toRemove := flags.Lookup(flagContainerLabelRemove).Value.(*opts.ListOpts).GetAll()
+		toRemove := flags.Lookup(flagContainerLabelRemove).Value.(*opts.ListOpts).GetSlice()
 		for _, label := range toRemove {
 			delete(*field, label)
 		}
@@ -696,7 +693,7 @@ func updateContainerLabels(flags *pflag.FlagSet, field *map[string]string) {
 			*field = map[string]string{}
 		}
 
-		values := flags.Lookup(flagContainerLabelAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagContainerLabelAdd).Value.(*opts.ListOpts).GetSlice()
 		for key, value := range opts.ConvertKVStringsToMap(values) {
 			(*field)[key] = value
 		}
@@ -705,7 +702,7 @@ func updateContainerLabels(flags *pflag.FlagSet, field *map[string]string) {
 
 func updateLabels(flags *pflag.FlagSet, field *map[string]string) {
 	if *field != nil && flags.Changed(flagLabelRemove) {
-		toRemove := flags.Lookup(flagLabelRemove).Value.(*opts.ListOpts).GetAll()
+		toRemove := flags.Lookup(flagLabelRemove).Value.(*opts.ListOpts).GetSlice()
 		for _, label := range toRemove {
 			delete(*field, label)
 		}
@@ -715,7 +712,7 @@ func updateLabels(flags *pflag.FlagSet, field *map[string]string) {
 			*field = map[string]string{}
 		}
 
-		values := flags.Lookup(flagLabelAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagLabelAdd).Value.(*opts.ListOpts).GetSlice()
 		for key, value := range opts.ConvertKVStringsToMap(values) {
 			(*field)[key] = value
 		}
@@ -724,7 +721,7 @@ func updateLabels(flags *pflag.FlagSet, field *map[string]string) {
 
 func updateSysCtls(flags *pflag.FlagSet, field *map[string]string) {
 	if *field != nil && flags.Changed(flagSysCtlRemove) {
-		values := flags.Lookup(flagSysCtlRemove).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagSysCtlRemove).Value.(*opts.ListOpts).GetSlice()
 		for key := range opts.ConvertKVStringsToMap(values) {
 			delete(*field, key)
 		}
@@ -734,7 +731,7 @@ func updateSysCtls(flags *pflag.FlagSet, field *map[string]string) {
 			*field = map[string]string{}
 		}
 
-		values := flags.Lookup(flagSysCtlAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagSysCtlAdd).Value.(*opts.ListOpts).GetSlice()
 		for key, value := range opts.ConvertKVStringsToMap(values) {
 			(*field)[key] = value
 		}
@@ -748,7 +745,7 @@ func updateUlimits(flags *pflag.FlagSet, ulimits []*container.Ulimit) []*contain
 		newUlimits[ulimit.Name] = ulimit
 	}
 	if flags.Changed(flagUlimitRemove) {
-		values := flags.Lookup(flagUlimitRemove).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagUlimitRemove).Value.(*opts.ListOpts).GetSlice()
 		for key := range opts.ConvertKVStringsToMap(values) {
 			delete(newUlimits, key)
 		}
@@ -783,7 +780,7 @@ func updateEnvironment(flags *pflag.FlagSet, field *[]string) {
 		}
 
 		value := flags.Lookup(flagEnvAdd).Value.(*opts.ListOpts)
-		for _, v := range value.GetAll() {
+		for _, v := range value.GetSlice() {
 			envSet[envKey(v)] = v
 		}
 
@@ -925,7 +922,7 @@ func buildToRemoveSet(flags *pflag.FlagSet, flag string) map[string]struct{} {
 		return toRemove
 	}
 
-	toRemoveSlice := flags.Lookup(flag).Value.(*opts.ListOpts).GetAll()
+	toRemoveSlice := flags.Lookup(flag).Value.(*opts.ListOpts).GetSlice()
 	for _, key := range toRemoveSlice {
 		toRemove[key] = empty
 	}
@@ -990,7 +987,7 @@ func updateMounts(flags *pflag.FlagSet, mounts *[]mounttypes.Mount) error {
 
 func updateGroups(flags *pflag.FlagSet, groups *[]string) error {
 	if flags.Changed(flagGroupAdd) {
-		values := flags.Lookup(flagGroupAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagGroupAdd).Value.(*opts.ListOpts).GetSlice()
 		*groups = append(*groups, values...)
 	}
 	toRemove := buildToRemoveSet(flags, flagGroupRemove)
@@ -1025,7 +1022,7 @@ func updateDNSConfig(flags *pflag.FlagSet, config **swarm.DNSConfig) error {
 
 	nameservers := (*config).Nameservers
 	if flags.Changed(flagDNSAdd) {
-		values := flags.Lookup(flagDNSAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagDNSAdd).Value.(*opts.ListOpts).GetSlice()
 		nameservers = append(nameservers, values...)
 	}
 	nameservers = removeDuplicates(nameservers)
@@ -1040,7 +1037,7 @@ func updateDNSConfig(flags *pflag.FlagSet, config **swarm.DNSConfig) error {
 
 	search := (*config).Search
 	if flags.Changed(flagDNSSearchAdd) {
-		values := flags.Lookup(flagDNSSearchAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagDNSSearchAdd).Value.(*opts.ListOpts).GetSlice()
 		search = append(search, values...)
 	}
 	search = removeDuplicates(search)
@@ -1055,7 +1052,7 @@ func updateDNSConfig(flags *pflag.FlagSet, config **swarm.DNSConfig) error {
 
 	options := (*config).Options
 	if flags.Changed(flagDNSOptionAdd) {
-		values := flags.Lookup(flagDNSOptionAdd).Value.(*opts.ListOpts).GetAll()
+		values := flags.Lookup(flagDNSOptionAdd).Value.(*opts.ListOpts).GetSlice()
 		options = append(options, values...)
 	}
 	options = removeDuplicates(options)
@@ -1204,7 +1201,7 @@ type hostMapping struct {
 func updateHosts(flags *pflag.FlagSet, hosts *[]string) error {
 	var toRemove []hostMapping
 	if flags.Changed(flagHostRemove) {
-		extraHostsToRemove := flags.Lookup(flagHostRemove).Value.(*opts.ListOpts).GetAll()
+		extraHostsToRemove := flags.Lookup(flagHostRemove).Value.(*opts.ListOpts).GetSlice()
 		for _, entry := range extraHostsToRemove {
 			hostName, ipAddr, _ := strings.Cut(entry, ":")
 			toRemove = append(toRemove, hostMapping{IPAddr: ipAddr, Host: hostName})
@@ -1238,7 +1235,7 @@ func updateHosts(flags *pflag.FlagSet, hosts *[]string) error {
 
 	// Append new hosts (in SwarmKit format)
 	if flags.Changed(flagHostAdd) {
-		values := convertExtraHostsToSwarmHosts(flags.Lookup(flagHostAdd).Value.(*opts.ListOpts).GetAll())
+		values := convertExtraHostsToSwarmHosts(flags.Lookup(flagHostAdd).Value.(*opts.ListOpts).GetSlice())
 		newHosts = append(newHosts, values...)
 	}
 	*hosts = removeDuplicates(newHosts)
@@ -1263,7 +1260,7 @@ func updateLogDriver(flags *pflag.FlagSet, taskTemplate *swarm.TaskSpec) error {
 
 	taskTemplate.LogDriver = &swarm.Driver{
 		Name:    name,
-		Options: opts.ConvertKVStringsToMap(flags.Lookup(flagLogOpt).Value.(*opts.ListOpts).GetAll()),
+		Options: opts.ConvertKVStringsToMap(flags.Lookup(flagLogOpt).Value.(*opts.ListOpts).GetSlice()),
 	}
 
 	return nil
@@ -1472,14 +1469,14 @@ func updateCapabilities(flags *pflag.FlagSet, containerSpec *swarm.ContainerSpec
 		capAdd  = opts.CapabilitiesMap(containerSpec.CapabilityAdd)
 	)
 	if flags.Changed(flagCapAdd) {
-		toAdd = opts.CapabilitiesMap(flags.Lookup(flagCapAdd).Value.(*opts.ListOpts).GetAll())
+		toAdd = opts.CapabilitiesMap(flags.Lookup(flagCapAdd).Value.(*opts.ListOpts).GetSlice())
 		if toAdd[opts.ResetCapabilities] {
 			capAdd = make(map[string]bool)
 			delete(toAdd, opts.ResetCapabilities)
 		}
 	}
 	if flags.Changed(flagCapDrop) {
-		toDrop = opts.CapabilitiesMap(flags.Lookup(flagCapDrop).Value.(*opts.ListOpts).GetAll())
+		toDrop = opts.CapabilitiesMap(flags.Lookup(flagCapDrop).Value.(*opts.ListOpts).GetSlice())
 		if toDrop[opts.ResetCapabilities] {
 			capDrop = make(map[string]bool)
 			delete(toDrop, opts.ResetCapabilities)
